// stores/useAlarmDataStore.js
import { defineStore } from "pinia";
import { ref } from "vue";

const useAlarmDataStore = defineStore("alarmData", () => {
  const alarmData = ref([]);
  const subscribers = ref([]); // 用於儲存 Subscriber 對象
  // 建立 alarmData 的函數
  const createAlarmData = (alarmList) => {
    alarmData.value = alarmList.children.map((item, index) => ({
      key: item.key,
      name: item.name || `Alarm ${index + 1}`,
      alarmCount: 0,
      unackedCount: 0,
      alarmOrd: item.children && item.children[0] ? item.children[0].ord : null, // 儲存 alarmOrd
      Ord: item.ord ? item.ord : null,
    }));
    // 在 store 初始化時，進行訂閱
    subscribeToAlarms();
  };

  // 更新特定 alarm 數據的函數
  const updateAlarmItem = (index, alarmCount, unackedCount) => {
    if (index >= 0 && index < alarmData.value.length) {
      alarmData.value.splice(index, 1, {
        ...alarmData.value[index],
        alarmCount,
        unackedCount,
      });
    }
  };
  const subscribeToAlarms = () => {
    if (typeof window.requirejs === "undefined") return;
    window.requirejs(["baja!"], (baja) => {
      // 對每個 alarm 項目建立訂閱
      alarmData.value.forEach((alarm, index) => {
        if (!alarm.alarmOrd) return;

        //  建立訂閱器
        const subscriber = new baja.Subscriber();

        // 定義 changed 事件的處理函數
        subscriber.attach("changed", (prop) => {
          try {
            let alarmCount = alarmData.value[index].alarmCount;
            let unackedCount = alarmData.value[index].unackedCount;

            if (prop.$getDisplayName() === "In Alarm Count") {
              // 取得 In Alarm Count
              alarmCount = prop.$getValue();
            }
            if (prop.$getDisplayName() === "Unacked Alarm Count") {
              // 取得 Unacked Alarm Count
              unackedCount = prop.$getValue();
            }

            // 更新 alarmData
            updateAlarmItem(index, alarmCount, unackedCount);
          } catch (error) {
            console.error(
              `處理 ${alarm.name || index} 告警變化失敗: ${error.message}`,
              error
            );
          }
        });

        // 訂閱 alarm 資訊
        baja.Ord.make(alarm.alarmOrd)
          .get({ subscriber })
          .then((result) => {
            console.log(`Successfuly subscribed to alarm ${alarm.name}`);
          })
          .catch((err) => {
            console.error(
              `訂閱 Alarm ${alarm.name || index} 失敗: ${err.message}`
            );
            subscriber.detach("changed");
          });
        subscribers.value.push(subscriber);
      });
    });
  };

  const clearAllSubscriber = () => {
    subscribers.value.forEach((subscriber) => {
      subscriber.detach("changed");
    });
    subscribers.value = [];
  };

  return { alarmData, createAlarmData, updateAlarmItem, clearAllSubscriber };
});

export default useAlarmDataStore;
