import { defineStore } from "pinia";
import { ref } from "vue";

const useUserStore = defineStore("user", () => {
  const userName = ref("webUser");
  const userRole = ref("");

  const loadUserInfo = () => {
    return new Promise((resolve, reject) => {
      if (window.require && window.requirejs) {
        window.requirejs(["baja!"], (baja) => {
          let currentUserName = baja.getUserName();
          userName.value = currentUserName;
          baja.Ord.make(
            `station:|slot:/Services/UserService/${currentUserName}`
          )
            .get()
            .then((user) => {
              const rolesString = user.getRoles(); // 取得角色字串
              if (rolesString) {
                const rolesArray = rolesString.split(",");
                if (rolesArray.length > 0) {
                  userRole.value = rolesArray[0].trim();
                  console.log("選取的角色:", userRole.value);
                }
              }
              resolve();
            })
            .catch((err) => {
              console.error(`訂閱失敗: ${err.message}`);
              reject(err);
            });
        });
      } else {
        console.error("未能載入 Baja 環境，請確認依賴已正確加載。");
      }
    });
  };

  return { userName, userRole, loadUserInfo };
});

export default useUserStore;
